"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isConstVar = exports.isResourceType = exports.isFunctionType = exports.isResourceVar = exports.getLocationOfNode = exports.isPrimitive = void 0;
const typescript_1 = __importDefault(require("typescript"));
const imports_1 = require("../imports");
const constants_1 = require("../constants");
function isPrimitive(val) {
    const primitiveTypes = [
        "string",
        "number",
        "boolean",
        "undefined",
        "symbol",
        "bigint",
        "void",
        "any",
        "unknown",
    ];
    return primitiveTypes.indexOf(val) !== -1;
}
exports.isPrimitive = isPrimitive;
function getLocationOfNode(node, depth = -1) {
    const sourceFile = node.getSourceFile();
    const startPos = typescript_1.default.getLineAndCharacterOfPosition(sourceFile, node.getStart());
    const startPosStr = `(${startPos.line},${startPos.character})`;
    const endPos = typescript_1.default.getLineAndCharacterOfPosition(sourceFile, node.getEnd());
    const endPosStr = `(${endPos.line},${endPos.character})`;
    const loc = {
        file: sourceFile.fileName,
        depth: depth,
        start: startPosStr,
        end: endPosStr,
    };
    return loc;
}
exports.getLocationOfNode = getLocationOfNode;
/**
 * Check if a variable is a resource.
 */
function isResourceVar(node, checker) {
    const type = checker.getTypeAtLocation(node);
    if (type == undefined) {
        return false;
    }
    const decls = type.symbol?.declarations;
    return (decls != undefined &&
        (typescript_1.default.isClassDeclaration(decls[0]) || typescript_1.default.isInterfaceDeclaration(decls[0])) &&
        isResourceType(decls[0], checker));
}
exports.isResourceVar = isResourceVar;
/**
 * Check if a type is function.
 */
function isFunctionType(type) {
    const flags = typescript_1.default.TypeFlags.Any |
        typescript_1.default.TypeFlags.Unknown |
        typescript_1.default.TypeFlags.Void |
        typescript_1.default.TypeFlags.Undefined |
        typescript_1.default.TypeFlags.Null |
        typescript_1.default.TypeFlags.Never |
        typescript_1.default.TypeFlags.Intersection |
        typescript_1.default.TypeFlags.Union |
        typescript_1.default.TypeFlags.Index;
    return type.getCallSignatures().length > 0 && (type.getFlags() & flags) === 0;
}
exports.isFunctionType = isFunctionType;
/**
 * Check if the declaration implements or extends the Resource interaface.
 * First, analyze the import dependencies of the source file for the current type node.
 * Then, check if the base types of this declaration are from the Resource interface in the plutolang base package.
 * If not, recursively check each base type.
 */
function isResourceType(declNode, checker, fnResource = false) {
    const targetTypeName = fnResource ? constants_1.FN_RESOURCE_TYPE_NAME : constants_1.RESOUCE_TYPE_NAME;
    // Analyze the import dependencies of the source file for the current type node.
    // If the type node implements the Resource interface, we can use this analysis result to
    // verify if the Resource interface belongs to plutolang.
    const importStore = new imports_1.ImportStore();
    const sourceFile = declNode.getSourceFile();
    const importStats = sourceFile.statements
        .filter(typescript_1.default.isImportDeclaration)
        .flatMap((stmt) => (0, imports_1.extractImportElements)(sourceFile, stmt));
    importStore.update(importStats);
    let found = false;
    declNode.heritageClauses?.forEach((clause) => {
        if (found) {
            return;
        }
        // Check every type in the implementation and extension clauses.
        for (const typeNode of clause.types) {
            const identifier = typeNode.expression;
            if (identifier.getText() == targetTypeName) {
                const elem = importStore.searchElement(targetTypeName);
                if (elem?.package == constants_1.PLUTO_BASE_PKG) {
                    found = true;
                    return;
                }
            }
            // Obtain the declaration of the base type. Then, proceed to recursively verify
            // if it implements or extends the Resource interface.
            const symbol = checker.getSymbolAtLocation(identifier);
            if (symbol == undefined || symbol.declarations == undefined) {
                continue;
            }
            for (const decl of symbol.declarations) {
                if (!typescript_1.default.isClassDeclaration(decl) && !typescript_1.default.isInterfaceDeclaration(decl)) {
                    continue;
                }
                if (isResourceType(decl, checker, fnResource)) {
                    found = true;
                    return;
                }
            }
        }
    });
    return found;
}
exports.isResourceType = isResourceType;
/**
 * Check if a symbol is a constant variable.
 */
function isConstVar(symbol) {
    const symbolDeclaration = symbol.valueDeclaration || symbol.declarations?.[0];
    return (!!symbolDeclaration &&
        typescript_1.default.isVariableDeclaration(symbolDeclaration) &&
        typescript_1.default.isVariableDeclarationList(symbolDeclaration.parent) &&
        symbolDeclaration.parent.flags === typescript_1.default.NodeFlags.Const);
}
exports.isConstVar = isConstVar;
//# sourceMappingURL=type-checker.js.map